Attribute VB_Name = "modValues"
'##### Modul zur Ausgabe der Simulationsergebnisse #####

Option Explicit

Dim n As Integer, m As Integer

Private Const cParamUnits = 8

Public Enum tValues1
    vbInstanz = 0
    vbVerzweigung = 1
    vbZelle = 2
End Enum

Public Enum tValues2
    vbSpannung = 0
    vbWiderstand = 1
    vbStromstrke = 2
    vbLeistung = 3
End Enum

Public Enum tValues3
    vbSchein = 0
    vbWirk = 1
    vbBlind = 2
    vbPhi = 3
End Enum

'Den gewnschten Wert als Gleitkommazahl zurckgeben
Public Function GetValDbl(Zelle As Integer, ByVal Area As tValues1, ByVal Unit As tValues2, ByVal Register As tValues3, Optional oldVal As Boolean) As Double
Dim Val1 As Complex, R As Complex, I As Complex
If oldVal = True Then
    R.Re = bZelle(Zelle).R_old(Area)
    I.Re = bZelle(Zelle).I_old(Area)
Else
    R = bZelle(Zelle).R(Area)
    I = bZelle(Zelle).I(Area)
End If
Select Case Unit
    Case 0 'Spannung U = R * I
        Val1 = MultiplyC(R, I)
    Case 1 'Widerstand
        Val1 = R
    Case 2 'Stromstrke
        Val1 = I
    Case 3 'Leistung P = U * I_konjugiert
        Val1 = MultiplyC(MultiplyC(R, I), ConjugateC(I))
End Select
Select Case Register
    Case 0
        If R.Re < 0 And Unit = vbWiderstand Then
            GetValDbl = -1
        Else
            GetValDbl = AbsC(Val1)
        End If
    Case 1
        GetValDbl = Val1.Re
    Case 2
        GetValDbl = Val1.Im
    Case 3
        If R.Re >= 0 Then GetValDbl = GetPhi(Val1) 'Phasenwinkel fr Durchgebranntes ist Unsinn
End Select
End Function

'Den gewnschten Wert wohlformiert zurckgeben
Public Function GetValStr(Zelle As Integer, ByVal Area As tValues1, ByVal Unit As tValues2, ByVal Register As tValues3, Optional oldVal As Boolean) As String
If Register <= 2 Then
    GetValStr = FormatData(GetValDbl(Zelle, Area, Unit, Register, oldVal), GetEinheitByUnit(Unit))
ElseIf Register = 3 Then
    GetValStr = Format(GetValDbl(Zelle, Area, Unit, Register, oldVal), "0.00") & ""
End If
End Function

'Wandelt die Unit-Konstante in eine Einheit um: vbSpannung = "V"
Public Function GetEinheitByUnit(ByVal Unit As tValues2) As String
Select Case Unit
    Case vbSpannung
        GetEinheitByUnit = "V"
    Case vbWiderstand
        GetEinheitByUnit = "Ohm"
    Case vbStromstrke
        GetEinheitByUnit = "A"
    Case vbLeistung
        GetEinheitByUnit = "W"
End Select
End Function

'Wandelt die Unit Konstante in den Namen um: vbSpannung = "Spannung"
Public Function GetNameByUnit(ByVal Unit As tValues2) As String
Select Case Unit
    Case vbSpannung
        GetNameByUnit = "Spannung"
    Case vbWiderstand
        GetNameByUnit = "Widerstand"
    Case vbStromstrke
        GetNameByUnit = "Stromstrke"
    Case vbLeistung
        GetNameByUnit = "Leistung"
End Select
End Function

'Wandelt die Unit Konstante in das Symbol um: vbSpannung = U
Public Function GetSymbolByUnit(ByVal Unit As tValues2) As String
Select Case Unit
    Case vbSpannung
        GetSymbolByUnit = "U"
    Case vbWiderstand
        GetSymbolByUnit = "R"
    Case vbStromstrke
        GetSymbolByUnit = "I"
    Case vbLeistung
        GetSymbolByUnit = "P"
End Select
End Function

'Wandelt die Area Konstante in den namen um: vbInstanz = "Instanz"
Public Function GetNameByArea(ByVal Area As tValues1) As String
Select Case Area
    Case vbInstanz
        GetNameByArea = "Instanz"
    Case vbVerzweigung
        GetNameByArea = "Zweig"
    Case vbZelle
        GetNameByArea = "Zelle"
End Select
End Function

'Gibt einen bestimmten Teil eines Parameters zurck
'SpecOutput -> 0: keine Einheit; 1: Nur Einheit; 2: Nur Symbol (R fr Ws.)
'NoSpecUnits: Einheiten [geschlossen = 1] NICHT
Public Function ModifyParam(strParam As String, ByVal SpecOutput As Byte, Optional ByVal NoSpecUnits As Byte) As String
If SpecOutput = 0 Then
    If Left(strParam, 1) = "[" Then Exit Function
    For n = Len(strParam) To 1 Step -1
        If Mid(strParam, n, 1) = "[" Then
            ModifyParam = Left(strParam, n - 2)
            Exit For
        End If
    Next
Else
    If Mid(strParam, 1, 1) = "[" And NoSpecUnits = 1 Then Exit Function
    For n = Len(strParam) To 1 Step -1
        If Mid(strParam, n, 1) = "[" Then
            ModifyParam = Mid(strParam, n + 1, Len(strParam) - n - 1)
            If SpecOutput = 2 Then
                Select Case ModifyParam
                    Case "A"
                        ModifyParam = "I"
                    Case "V"
                        ModifyParam = "U"
                    Case "W"
                        ModifyParam = "P"
                    Case "Ohm"
                        ModifyParam = "R"
                    Case "Hz"
                        ModifyParam = "f"
                    Case "F"
                        ModifyParam = "C"
                    Case "H"
                        ModifyParam = "L"
                End Select
            End If
            Exit For
        End If
    Next
End If
End Function

'Funktion zum Formatieren von Simulationsergebnissen (z. B. (1432, "V") = "1,43kV" )
'SpecOutput -> 0: Wert + Einheit; 1: nur Wert; 2: nur Einheit
'Exponent -> ByRef Wert, um den Exponenten zu bergeben (-1) oder zwingend zu verwendet (>=0)
Function FormatData(ByVal Expression As Double, Einheit As String, Optional SpecOutput As Byte, Optional MaxGZ As Integer, Optional ByRef Exponent As Integer = -1, Optional ByVal MaxDigits As Integer) As String
Dim nExp As Integer, valNK As String, Vorzeichen As Integer, aktGZ As Byte
'Gltige Ziffern festlegen
If MaxGZ > 0 And MaxGZ < GZ Then
    aktGZ = MaxGZ
Else
    aktGZ = GZ
End If
If Expression = 0 Then GoTo a
Vorzeichen = Sgn(Expression)
Expression = Abs(Expression)
Do Until (Abs(Expression) >= 1 And Abs(Expression) < 1000) Or (nExp = Exponent And Exponent <> -1)
    If Abs(Expression) < 1 Then   'Kleiner Wert
        Expression = Expression * 1000
        nExp = nExp - 3 'Komma drei Stellen nach rechts
    Else                     'Groer Wert
        Expression = Expression / 1000
        nExp = nExp + 3 'Komma drei stellen nach links
    End If
    If Abs(nExp) > 15 Then
        Expression = 0
        nExp = 0
        GoTo a
    End If
Loop
If aktGZ < (Len(CStr(Int(Expression)))) Then   'Nach links wegen gltigen Ziffern (z.B. 721V = 0,72KV)
    Expression = Expression / 1000
    nExp = nExp + 3
End If
a:
If Exponent = -1 Then Exponent = nExp
If SpecOutput = 2 Then FormatData = GetDecimal(nExp) & Einheit: Exit Function
If MaxGZ >= 0 Then
    valNK = String(aktGZ - Len(CStr(Int(Expression))) + ((Left(CStr(Expression), 1) = 0) And Expression > 0) * -1, "0")
    If valNK = "" Then
        FormatData = Format(Expression * Vorzeichen, "0")
    Else
        FormatData = Format(Expression * Vorzeichen, "0." & valNK)
    End If
Else 'Keine GZ beachten
    FormatData = CStr(Expression * Vorzeichen)
End If
If MaxDigits > 0 And Len(FormatData) > MaxDigits Then FormatData = FormatData(Expression, Einheit, 1, 1)
If SpecOutput = 0 Then FormatData = FormatData & " " & GetDecimal(nExp) & Einheit
End Function

'Param als wohlformatierten Wert zurckgeben
Function GetParamStr(Zelle As Integer, ParamNr As Byte) As String
Dim ParamVal As Double
If ParamNr = 1 Then
    GetParamStr = FormatData(SngToDbl(aZelle(Zelle).Param1), ModifyParam(IDtoParam1(aZelle(Zelle).ID), 1, 1), , -1)
Else
    GetParamStr = FormatData(SngToDbl(aZelle(Zelle).Param2), ModifyParam(IDtoParam2(aZelle(Zelle).ID), 1, 1), , -1)
End If
End Function

'Formatierten Parmater in eine Zahl umwandeln "1,2 kOhm" = 1200)
Function ParseParam(ByVal Expression As String, ByRef ParamVal As Single) As Boolean
On Error GoTo ende
Dim Pos As Integer, aktVal As Single
'Zahl isolieren
Pos = 0
Do
    Pos = Pos + 1
    If IsNumeric(Mid(Expression, 1, Pos)) = False Then
        'Auf eventuellen Exponent berprfen
        If IsNumeric(Mid(Expression, 1, Pos + 1)) = False And IsNumeric(Mid(Expression, 1, Pos + 2)) = False Then
            aktVal = Mid(Expression, 1, Pos - 1)
            Exit Do
        End If
    ElseIf Pos = Len(Expression) Then
        aktVal = Mid(Expression, 1, Pos)
        Pos = Pos + 1
        Exit Do
    End If
Loop
'Restlichen Teil isolieren
ParamVal = aktVal
If Len(Expression) > (Pos - 1) Then
    Expression = Mid(Expression, Pos, Len(Expression) - Pos + 1)
    Expression = Replace(Expression, " ", "") 'Leerzeichen rauswerfen
    For n = 1 To cParamUnits      'Einheit rauswerfen, falls vorhanden
         m = InStr(Expression, GetUnitByNr(n))
         If m > 0 Then
            Expression = Mid(Expression, 1, m - 1)
            Exit For
        End If
    Next
    'Nach Dezimalvorsatz suchen
    If Len(Expression) > 1 Then Exit Function 'Unzulssig
    If Len(Expression) = 1 Then
        If GetExponent(Expression) = 0 Then
            Exit Function
        Else
            ParamVal = ParamVal * 10 ^ GetExponent(Expression)
        End If
    End If
End If
ParseParam = True
ende:
End Function

'Function um bei gegebenen Exponent den Dezimalvorsatz zu erhalten (10 hoch 12 = Tera) (Stcker Physik S. 1031)
Function GetDecimal(Exponent As Integer) As String
Select Case Exponent
    Case -15
        GetDecimal = "f"
    Case -12
        GetDecimal = "p"
    Case -9
        GetDecimal = "n"
    Case -6
        GetDecimal = ""
    Case -3
        GetDecimal = "m"
    Case 3
        GetDecimal = "k"
    Case 6
        GetDecimal = "M"
    Case 9
        GetDecimal = "G"
    Case 12
        GetDecimal = "T"
    Case 15
        GetDecimal = "P"
End Select
End Function

'Umkehrfunktion zu GetDecimal
Function GetExponent(Expression As String) As Integer
Select Case Expression
    Case "f"
        GetExponent = -15
    Case "p"
        GetExponent = -12
    Case "n"
        GetExponent = -9
    Case ""
        GetExponent = -6
    Case "m"
        GetExponent = -3
    Case "c"
        GetExponent = -2
    Case "d"
        GetExponent = -1
    Case "k"
        GetExponent = 3
    Case "M"
        GetExponent = 6
    Case "G"
        GetExponent = 9
    Case "T"
        GetExponent = 12
    Case "P"
        GetExponent = 15
End Select
End Function

'Einheit herausfinden (fr Parsen) / die lngste zuerst
Function GetUnitByNr(Nr As Integer) As String
Select Case Nr
    Case 1
        GetUnitByNr = "Ohm"
    Case 2
        GetUnitByNr = "Hz"
    Case 3
        GetUnitByNr = "V"
    Case 4
        GetUnitByNr = "W"
    Case 5
        GetUnitByNr = "A"
    Case 6
        GetUnitByNr = "F"
    Case 7
        GetUnitByNr = "H"
    Case 8
        GetUnitByNr = "s"
End Select
End Function
